@@ -1,3 +1,7 @@
+# pylint: disable=missing-function-docstring
+# pylint: disable=missing-class-docstring
+# pylint: disable=line-too-long
+# pylint: disable=unused-import
 '''
     A module of utility methods used for formatting strings.
 
@@ -7,25 +11,31 @@
     ----------
     `author`: Colemen Atwood
     `created`: 12-09-2021 09:00:27
-    `memberOf`: string_conversion
-    `version`: 1.0
-    `method_name`: string_conversion
+    `memberOf`: string_utils
 '''
 
 
 # import json
 # import hashlib
-import logging
-from typing import Union
-import re
-import os
+import logging as _logging
+from typing import Union as _Union
+import re as _re
+import os as _os
 import urllib.parse
 
-from utils.dict_utils.dict_utils import get_kwarg as _get_kwarg
+import utils.dict_utils as _obj
+# from pyparsing import And, Suppress,Dict, Word, Literal, Group, Optional, ZeroOrMore, OneOrMore, Regex, restOfLine, alphanums,nums, printables, string, CaselessKeyword,nestedExpr,ParseException,quotedString,removeQuotes,originalTextFor,delimitedList,QuotedString
+# from pyparsing import QuotedString
 
-logger = logging.getLogger(__name__)
 
 
+from utils.string_utils.string_escaping import escape_charcode
+from utils.string_utils.string_escaping import escape_quoted_chars
+from utils.string_utils.string_escaping import escape_regex
+from utils.string_utils.string_strip import strip_end
+
+logger = _logging.getLogger(__name__)
+
 
 def to_pascal_case(subject:str,first_char_lower=False,first_char_alpha=True)->str:
     '''
@@ -59,19 +69,19 @@ def to_pascal_case(subject:str,first_char_lower=False,first_char_alpha=True)->st
     '''
 
 
-    subject = re.sub(r'[^a-zA-Z0-9_\s]', ' ', subject)
+    subject = _re.sub(r'[^a-zA-Z0-9_\s]', ' ', subject)
 
     subject = to_snake_case(subject)
-    subject = re.sub(r'([a-zA-Z0-9])([a-zA-Z0-9]*)', lambda x: x.group(1).upper() + x.group(2).lower(), subject)
+    subject = _re.sub(r'([a-zA-Z0-9])([a-zA-Z0-9]*)', lambda x: x.group(1).upper() + x.group(2).lower(), subject)
 
     if first_char_alpha:
-        subject = re.sub(r'^[^a-zA-Z]*', r'', subject)
+        subject = _re.sub(r'^[^a-zA-Z]*', r'', subject)
     else:
         first_char_lower=False
-    subject = re.sub(r'_', '', subject)
+    subject = _re.sub(r'_', '', subject)
 
     if first_char_lower:
-        subject = re.sub(r'^([A-Za-z])', lambda x: x.group(1).upper(), subject)
+        subject = _re.sub(r'^([A-Za-z])', lambda x: x.group(1).upper(), subject)
 
     return subject
 
@@ -105,19 +115,19 @@ def to_camel_case(subject:str,first_char_lower=True,first_char_alpha=True)->str:
         `method_name`: to_camel_case
         * @xxx [06-03-2022 09:26:00]: documentation for to_camel_case
     '''
-    subject = re.sub(r'[^a-zA-Z0-9_\s]', ' ', subject)
+    subject = _re.sub(r'[^a-zA-Z0-9_\s]', ' ', subject)
 
     subject = to_snake_case(subject)
-    subject = re.sub(r'([a-zA-Z0-9])([a-zA-Z0-9]*)', lambda x: x.group(1).upper() + x.group(2).lower(), subject)
+    subject = _re.sub(r'([a-zA-Z0-9])([a-zA-Z0-9]*)', lambda x: x.group(1).upper() + x.group(2).lower(), subject)
 
     if first_char_alpha:
-        subject = re.sub(r'^[^a-zA-Z]*', r'', subject)
+        subject = _re.sub(r'^[^a-zA-Z]*', r'', subject)
     else:
         first_char_lower=False
-    subject = re.sub(r'_', '', subject)
+    subject = _re.sub(r'_', '', subject)
 
     if first_char_lower:
-        subject = re.sub(r'^([A-Za-z])', lambda x: x.group(1).lower(), subject)
+        subject = _re.sub(r'^([A-Za-z])', lambda x: x.group(1).lower(), subject)
 
     return subject
 
@@ -147,11 +157,11 @@ def to_snake_case(subject:str,first_char_alpha=False)->str:
     subject = str(subject)
 
     if first_char_alpha:
-        subject = re.sub(r'^[^a-zA-Z]*',"",subject)
+        subject = _re.sub(r'^[^a-zA-Z]*',"",subject)
 
-    subject = re.sub(r'[^a-zA-Z0-9_\s]'," ",subject)
-    subject = re.sub(r'(\s|_)+',"_",subject)
-    subject = re.sub(r'([a-z])(?:\s|_)?([A-Z])',r"\1_\2",subject)
+    subject = _re.sub(r'[^a-zA-Z0-9_\s]'," ",subject)
+    subject = _re.sub(r'(\s|_)+',"_",subject)
+    subject = _re.sub(r'([a-z])(?:\s|_)?([A-Z])',r"\1_\2",subject)
     return subject.lower()
 
 def to_screaming_snake(subject:str):
@@ -235,7 +245,7 @@ def to_dot_notation(subject:str)->str:
 
     subject = to_snake_case(subject)
     dotted = subject.replace("_",".")
-    output = re.sub(r'^\.*','',dotted)
+    output = _re.sub(r'^\.*','',dotted)
     return output
 
 def to_title_case(value:str,reverse=False)->str:
@@ -374,7 +384,6 @@ def limit_length(subject, max_len=2, from_beginning=True):
         if from_beginning is False:
             return subject[max_len:]
 
-
 def leftPad(subject, max_len=2, pad_char='0', limit_len=False, from_beginning=True):
     '''
         Convert a subject to snake_case.
@@ -414,7 +423,6 @@ def leftPad(subject, max_len=2, pad_char='0', limit_len=False, from_beginning=Tr
         subject = f"{pad_char * (max_len - slen)}{subject}"
     return subject
 
-
 def rightPad(subject, max_len=2, pad_char='0'):
     '''
         Convert a subject to snake_case.
@@ -443,7 +451,6 @@ def rightPad(subject, max_len=2, pad_char='0'):
         subject = f"{subject}{pad_char * (max_len - slen)}"
     return subject
 
-
 def encode_quotes(value:str)->str:
     '''
         Encodes single and double quotes within the value to &apos; or &quot; respectively.
@@ -471,7 +478,6 @@ def encode_quotes(value:str)->str:
     value = value.replace('"', "&quot;")
     return value
 
-
 def decode_quotes(value:str)->str:
     '''
         Decodes single and double quotes within the value from &apos; or &quot; respectively.
@@ -500,166 +506,166 @@ def decode_quotes(value:str)->str:
     return value
 
 
-def strip_excessive_spaces(value:str)->str:
-    '''
-        Removes excessive (2 or more consecutive) spaces from the string.
+# def strip_excessive_spaces(value:str)->str:
+#     '''
+#         Removes excessive (2 or more consecutive) spaces from the string.
 
-        ----------
+#         ----------
 
-        Arguments
-        -------------------------
-        `value` {str}
-            The string to format.
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to format.
 
-        Return {str}
-        ----------------------
-        The formatted string
+#         Return {str}
+#         ----------------------
+#         The formatted string
 
-        Meta
-        ----------
-        `author`: Colemen Atwood
-        `created`: 12-09-2021 08:19:28
-        `memberOf`: string_format
-        `version`: 1.0
-        `method_name`: strip_excessive_spaces
-    '''
-    return re.sub(r'[\s\s]{2,}', ' ', value)
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 12-09-2021 08:19:28
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: strip_excessive_spaces
+#     '''
+#     return _re.sub(r'[\s\s]{2,}', ' ', value)
 
 
-def strip_excessive_chars(value:str,chars:Union[str,list])->str:
-    '''
-        Removes excessive (2 or more consecutive) chars from the string.
+# def strip_excessive_chars(value:str,chars:_Union[str,list])->str:
+#     '''
+#         Removes excessive (2 or more consecutive) chars from the string.
 
-        ----------
+#         ----------
 
-        Arguments
-        -------------------------
-        `value` {str}
-            The string to format.
-        `chars` {str|list}
-            The chars to remove if they occur excessively.
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to format.
+#         `chars` {str|list}
+#             The chars to remove if they occur excessively.
 
-        Return {str}
-        ----------------------
-        The formatted string
+#         Return {str}
+#         ----------------------
+#         The formatted string
 
-        Meta
-        ----------
-        `author`: Colemen Atwood
-        `created`: 06-03-2022 11:47:37
-        `memberOf`: string_format
-        `version`: 1.0
-        `method_name`: strip_excessive_chars
-    '''
-    if isinstance(chars,(str)):
-        chars = [chars]
-    for c in chars:
-        if c == " ":
-            c = "\\s"
-        reg_c = escape_regex(c)
-        exp = rf"[{reg_c}]{{2,}}"
-        # print(exp)
-        reg = re.compile(exp)
-        value = re.sub(reg, c, value)
-    return value
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-03-2022 11:47:37
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: strip_excessive_chars
+#     '''
+#     if isinstance(chars,(str)):
+#         chars = [chars]
+#     for c in chars:
+#         if c == " ":
+#             c = "\\s"
+#         reg_c = escape_regex(c)
+#         exp = rf"[{reg_c}]{{2,}}"
+#         # print(exp)
+#         reg = _re.compile(exp)
+#         value = _re.sub(reg, c, value)
+#     return value
 
 
-def escape_regex(value:str)->str:
-    '''
-        Escapes regex special characters.
+# def escape_regex(value:str)->str:
+#     '''
+#         Escapes regex special characters.
 
-        ----------
+#         ----------
 
-        Arguments
-        -------------------------
-        `value` {str}
-            The string to escape.
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to escape.
 
-        Return {str}
-        ----------------------
-        The formatted string.
+#         Return {str}
+#         ----------------------
+#         The formatted string.
 
-        Meta
-        ----------
-        `author`: Colemen Atwood
-        `created`: 12-09-2021 08:46:32
-        `memberOf`: string_format
-        `version`: 1.0
-        `method_name`: escape_regex
-    '''
-    regex_chars = ["\\", "^", "$", "{", "}", "[", "]", "(", ")", ".", "*", "+", "?", "<", ">", "-", "&"]
-    for char in regex_chars:
-        value = value.replace(char, f"\\{char}")
-    return value
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 12-09-2021 08:46:32
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: escape_regex
+#     '''
+#     regex_chars = ["\\", "^", "$", "{", "}", "[", "]", "(", ")", ".", "*", "+", "?", "<", ">", "-", "&"]
+#     for char in regex_chars:
+#         value = value.replace(char, f"\\{char}")
+#     return value
 
-def strip_any(value,chars,side='both'):
-    '''
-        Strips characters/strings from the beginning,end or both sides of a string.
+# def strip_any(value,chars,side='both'):
+#     '''
+#         Strips characters/strings from the beginning,end or both sides of a string.
 
-        ----------
+#         ----------
 
-        Arguments
-        -------------------------
-        `value` {str}
-            The value to be formatted
-        `chars` {list}
-            A list of strings to be stripped from the value
-        [`side`='both'] {str}
-            Can be "left", "right", "both"
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The value to be formatted
+#         `chars` {list}
+#             A list of strings to be stripped from the value
+#         [`side`='both'] {str}
+#             Can be "left", "right", "both"
 
-        Return {str}
-        ----------------------
-        The formatted value.
+#         Return {str}
+#         ----------------------
+#         The formatted value.
 
-        Meta
-        ----------
-        `author`: Colemen Atwood
-        `created`: 02-04-2022 10:25:04
-        `memberOf`: string_format
-        `version`: 1.0
-        `method_name`: strip_any
-    '''
-    new_value = value
-    cycle = True
-    if side in ["both","left"]:
-        cycle = True
-        while cycle is True:
-            matchFound = False
-            for char in chars:
-                if new_value.startswith(char):
-                    new_value = new_value[len(char):]
-                    matchFound = True
-                else:
-                    continue
-            if matchFound is False:
-                cycle = False
-    if side in ["both","right"]:
-        cycle = True
-        while cycle is True:
-            matchFound = False
-            for char in chars:
-                if new_value.endswith(char):
-                    new_value = new_value[:-len(char)]
-                    matchFound = True
-                else:
-                    continue
-            if matchFound is False:
-                cycle = False
-    return new_value
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 02-04-2022 10:25:04
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: strip_any
+#     '''
+#     new_value = value
+#     cycle = True
+#     if side in ["both","left"]:
+#         cycle = True
+#         while cycle is True:
+#             matchFound = False
+#             for char in chars:
+#                 if new_value.startswith(char):
+#                     new_value = new_value[len(char):]
+#                     matchFound = True
+#                 else:
+#                     continue
+#             if matchFound is False:
+#                 cycle = False
+#     if side in ["both","right"]:
+#         cycle = True
+#         while cycle is True:
+#             matchFound = False
+#             for char in chars:
+#                 if new_value.endswith(char):
+#                     new_value = new_value[:-len(char)]
+#                     matchFound = True
+#                 else:
+#                     continue
+#             if matchFound is False:
+#                 cycle = False
+#     return new_value
 
 
 
-def strip_end(text, suffix):
-    if suffix and text.endswith(suffix):
-        return text[:-len(suffix)]
-    return text
+# def strip_end(text, suffix):
+#     if suffix and text.endswith(suffix):
+#         return text[:-len(suffix)]
+#     return text
 
 
 def windows_file_name(file_name:str)->str:
     '''
         Strips (window's) invalid characters from the file_name.\n
         These characters are removed:\n
-        <>:"/\|?*
+        <>:"/\\|?*
 
         ----------
 
@@ -680,10 +686,9 @@ def windows_file_name(file_name:str)->str:
         `version`: 1.0
         `method_name`: windows_file_name
     '''
-    return re.sub(r'[<>:"/\|?*]*', "", file_name)
+    return _re.sub(r'[<>:"/\|?*]*', "", file_name)
 
-
-def extension(string:Union[str,list])->Union[str,list]:
+def extension(string:_Union[str,list])->_Union[str,list]:
     '''
         Formats a file extension to have no leading period.
 
@@ -721,7 +726,7 @@ def extension(string:Union[str,list])->Union[str,list]:
     for ext in string:
         # print(f"ext: {ext}")
         ext = ext.lower()
-        ext = re.sub(r"^\.*", '', ext)
+        ext = _re.sub(r"^\.*", '', ext)
         new_ext_array.append(ext)
 
     if len(new_ext_array) > 1:
@@ -772,23 +777,23 @@ def file_path(value:str, **kwargs)->str:
         `version`: 1.0
         `method_name`: file_path
     '''
-    strip_trailing_separator = _get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
-    treat_as_url = _get_kwarg(["url"], False, (bool), **kwargs)
-    escape_spaces = _get_kwarg(["escape spaces"], False, (bool), **kwargs)
-    path_sep = escape_regex(os.path.sep)
+    strip_trailing_separator = _obj.get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
+    treat_as_url = _obj.get_kwarg(["url"], False, (bool), **kwargs)
+    escape_spaces = _obj.get_kwarg(["escape spaces"], False, (bool), **kwargs)
+    path_sep = escape_regex(_os.path.sep)
     if treat_as_url is True:
         return url(value, **kwargs)
     # print(f"path_sep: {path_sep}")
-    value = re.sub(r'(\\|\/)', path_sep, value)
-    regex = re.compile(rf'{path_sep}+')
-    value = re.sub(regex, path_sep, value)
+    value = _re.sub(r'(\\|\/)', path_sep, value)
+    regex = _re.compile(rf'{path_sep}+')
+    value = _re.sub(regex, path_sep, value)
     # only clean the path for non urls
     if len(value) > 250:
         # print(f"clean path: ",value)
         value = clean_path(value)
         # print(f"clean path: ",value)
     if escape_spaces:
-        path_array = value.split(os.path.sep)
+        path_array = value.split(_os.path.sep)
         new_path_array = []
         for seg in path_array:
             seg_string = seg
@@ -799,7 +804,7 @@ def file_path(value:str, **kwargs)->str:
     if strip_trailing_separator is True:
         value = strip_end(value, path_sep)
     return value
-
+format_file_path = file_path
 
 def url(value:str, **kwargs)->str:
     '''
@@ -835,13 +840,13 @@ def url(value:str, **kwargs)->str:
         `version`: 1.0
         `method_name`: url
     '''
-    strip_trailing_separator = _get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
-    encode = _get_kwarg(["encode", "url_encode"], False, (bool), **kwargs)
-    decode = _get_kwarg(["decode", "url_decode"], False, (bool), **kwargs)
+    strip_trailing_separator = _obj.get_kwarg(["strip_trailing_separator"], True, (bool), **kwargs)
+    encode = _obj.get_kwarg(["encode", "url_encode"], False, (bool), **kwargs)
+    decode = _obj.get_kwarg(["decode", "url_decode"], False, (bool), **kwargs)
     path_sep = "/"
-    value = re.sub(r'(\\|\/)', path_sep, value)
-    regex = re.compile(rf'{path_sep}+')
-    value = re.sub(regex, path_sep, value)
+    value = _re.sub(r'(\\|\/)', path_sep, value)
+    regex = _re.compile(rf'{path_sep}+')
+    value = _re.sub(regex, path_sep, value)
     if encode is True:
         value = urllib.parse.quote(value)
     if decode is True:
@@ -851,16 +856,17 @@ def url(value:str, **kwargs)->str:
         value = strip_end(value, path_sep)
 
     return value
+format_url = url
 
 
 def clean_path(path):
-    path = path.replace('/', os.sep).replace('\\', os.sep)
-    path = re.sub(r'^[\\\?]*','',path)
-    if os.sep == '\\' and '\\\\?\\' not in path:
+    path = path.replace('/', _os.sep).replace('\\', _os.sep)
+    path = _re.sub(r'^[\\\?]*','',path)
+    if _os.sep == '\\' and '\\\\?\\' not in path:
         # fix for Windows 260 char limit
-        relative_levels = len([directory for directory in path.split(os.sep) if directory == '..'])
-        cwd = [directory for directory in os.getcwd().split(os.sep)] if ':' not in path else []
-        path = '\\\\?\\' + os.sep.join(cwd[:len(cwd)-relative_levels] + [directory for directory in path.split(os.sep) if directory != ''][relative_levels:])
+        relative_levels = len([directory for directory in path.split(_os.sep) if directory == '..'])
+        cwd = [directory for directory in _os.getcwd().split(_os.sep)] if ':' not in path else []
+        path = '\\\\?\\' + _os.sep.join(cwd[:len(cwd)-relative_levels] + [directory for directory in path.split(_os.sep) if directory != ''][relative_levels:])
         # path = path.replace("\\\\?\\?\\","\\\\?\\")
     path = path.replace("\\\\?\\?\\","\\\\?\\")
     return path
@@ -901,42 +907,42 @@ def strip_empty_lines(value:str)->str:
 
 
 
-def reverse_sanitize_quotes(string):
-    orig_list = False
-    if isinstance(string, (list)):
-        orig_list = True
-    if isinstance(string, (str)):
-        string = [string]
+# def reverse_sanitize_quotes(string):
+#     orig_list = False
+#     if isinstance(string, (list)):
+#         orig_list = True
+#     if isinstance(string, (str)):
+#         string = [string]
 
-    new_list = []
-    for item in string:
-        item = item.replace("&apos_", "'")
-        item = item.replace("&quot_", '"')
-        new_list.append(item)
+#     new_list = []
+#     for item in string:
+#         item = item.replace("&apos_", "'")
+#         item = item.replace("&quot_", '"')
+#         new_list.append(item)
 
-    if len(new_list) == 1 and orig_list is False:
-        return new_list[0]
+#     if len(new_list) == 1 and orig_list is False:
+#         return new_list[0]
 
-    return new_list
+#     return new_list
 
-def sanitize_quotes(string):
-    orig_list = False
-    if isinstance(string, (list)):
-        orig_list = True
-    if isinstance(string, (str)):
-        string = [string]
+# def sanitize_quotes(string):
+#     orig_list = False
+#     if isinstance(string, (list)):
+#         orig_list = True
+#     if isinstance(string, (str)):
+#         string = [string]
 
-    new_list = []
-    for item in string:
-        if isinstance(item, (str)):
-            item = item.replace("'", "&apos_")
-            item = item.replace('"', "&quot_")
-        new_list.append(item)
+#     new_list = []
+#     for item in string:
+#         if isinstance(item, (str)):
+#             item = item.replace("'", "&apos_")
+#             item = item.replace('"', "&quot_")
+#         new_list.append(item)
 
-    if len(new_list) == 1 and orig_list is False:
-        return new_list[0]
+#     if len(new_list) == 1 and orig_list is False:
+#         return new_list[0]
 
-    return new_list
+#     return new_list
 
 
 
@@ -1016,7 +1022,7 @@ def array_replace_string(value,needles, new_string="", **kwargs):
         `version`: 1.0
         `method_name`: array_replace_string
     '''
-    max_replace = _get_kwarg(['max','max replace'],0,(int),**kwargs)
+    max_replace = _obj.get_kwarg(['max','max replace'],0,(int),**kwargs)
     if len(needles) == 0:
         return value
 
@@ -1036,10 +1042,227 @@ def array_replace_string(value,needles, new_string="", **kwargs):
 
     return result_string
 
+# def _get_numeric_charcode(char):
+#     for _,v in _numeric_char_codes.items():
+#         for c,code in v.items():
+#             if c == char:
+#                 return code
+#     return None
+
+# def _get_numeric_charcode_chars(cat:str)->list:
+#     output = []
+#     if cat in _numeric_char_codes:
+#         for k,_ in _numeric_char_codes[cat].items():
+#             output.append(k)
+#     return output
+
+# def charcode_escape(value:str,**kwargs)->str:
+#     '''
+#         Escape character in a string with their character codes.
+
+#         ----------
+
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to escape.
+
+#         Keyword Arguments
+#         -------------------------
+#         `chars` {list|str}
+#             A list or string of characters to escape.
+#         [`symbols`=True] {bool}
+#             If True, all symbols will be escaped.
+#         [`numeric`=False] {bool}
+#             If True, all numeric will be escaped.
+#         [`upper_case`=False] {bool}
+#             If True, all upper_case will be escaped.
+#         [`lower_case`=False] {bool}
+#             If True, all lower_case will be escaped.
+
+#         Return {str}
+#         ----------------------
+#         The escaped string.
+
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-04-2022 09:36:42
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: charcode_escape
+#         * @xxx [06-04-2022 09:38:48]: documentation for charcode_escape
+#     '''
+
+
+#     chars = _obj.get_kwarg(["chars"], None, (list,str), **kwargs)
+#     symbols = _obj.get_kwarg(["symbols"], True, (bool), **kwargs)
+#     numeric = _obj.get_kwarg(["numeric"], False, (bool), **kwargs)
+#     upper_case = _obj.get_kwarg(["upper_case"], False, (bool), **kwargs)
+#     lower_case = _obj.get_kwarg(["lower_case"], False, (bool), **kwargs)
+
+#     if chars is not None:
+#         if isinstance(chars,(str)):
+#             chars = chars.split()
+#         for c in chars:
+#             code = _get_numeric_charcode(c)
+#             value = value.replace(c,code)
+#         return value
+
+#     chars = []
+
+#     if symbols is True:
+#         chars = chars + _get_numeric_charcode_chars('symbols')
+#     if numeric is True:
+#         chars = chars + _get_numeric_charcode_chars('numeric')
+#     if upper_case is True:
+#         chars = chars + _get_numeric_charcode_chars('upper_case')
+#     if lower_case is True:
+#         chars = chars + _get_numeric_charcode_chars('lower_case')
+
+#     for c in chars:
+#         code = _get_numeric_charcode(c)
+#         value = value.replace(c,code)
+
+#     return value
+
+# def charcode_unescape(value:str)->str:
+#     '''
+#         Unescape the character codes from charcode_escape.
+
+#         ----------
+
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to unescape.
+
+#         Return {str}
+#         ----------------------
+#         The unescaped string.
+
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-04-2022 09:39:02
+#         `memberOf`: string_format
+#         `version`: 1.0
+#         `method_name`: charcode_unescape
+#         * @xxx [06-04-2022 09:39:50]: documentation for charcode_unescape
+#     '''
+
+#     for _,v in _numeric_char_codes.items():
+#         for c,code in v.items():
+#             value = value.replace(code,c)
+#     return value
 
 
 
 
+# def escape_quoted_commas(value,escape_value="__ESCAPED_COMMA__",reverse=False):
+#     '''
+#         Escape commas that are located within quotes.
+
+#         ----------
+
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to search within
+#         [`escape_value`='__ESCAPED_COMMA__] {str}
+#             The value to replace commas with.
+#         `reverse` {bool}
+#             if True it will replace the escaped commas with actual commas.
+
+#         Return {str}
+#         ----------------------
+#         The string with escaped commas.
+
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-01-2022 06:54:48
+#         `memberOf`: parse_sql
+#         `version`: 1.0
+#         `method_name`: escape_quoted_commas
+#         @xxx [06-01-2022 06:57:45]: documentation for escape_quoted_commas
+#     '''
+
+#     if reverse is True:
+#         return value.replace(escape_value,",")
+
+#     sql_qs = QuotedString("'", esc_quote="''")
+#     quote = sql_qs.search_string(value)
+#     if len(quote) > 0:
+#         quote = quote.asList()
+#         # print(f"quote: {quote}")
+#         for q in quote:
+#             if len(q) == 1:
+#                 q = q[0]
+#             esc = q.replace(",",escape_value)
+#             value = value.replace(q,esc)
+
+#     # print(sql_qs.search_string(value))
+#     return value
+
+# def escape_quoted_chars(value,reverse=False):
+#     '''
+#         Escape characters that can effect parsing which are located within quotes.
+
+#         ----------
+
+#         Arguments
+#         -------------------------
+#         `value` {str}
+#             The string to search within
+
+#         `reverse` {bool}
+#             if True it will reverse the escaped chars with their actual chars.
+
+#         Return {str}
+#         ----------------------
+#         The string with escaped chars.
+
+#         Meta
+#         ----------
+#         `author`: Colemen Atwood
+#         `created`: 06-01-2022 06:54:48
+#         `memberOf`: parse_sql
+#         `version`: 1.0
+#         `method_name`: escape_quoted_chars
+#         @xxx [06-01-2022 06:57:45]: documentation for escape_quoted_chars
+#     '''
+
+#     escapes = [
+#         [",","__&#44__"],
+#         [";","__&#59__"],
+#         ["(","__&#40__"],
+#         [")","__&#41__"],
+#         ["`","__&#96__"],
+#         ['"',"__&#34__"],
+#         ["'","__&#39__"],
+#     ]
+
+
+#     if reverse is True:
+#         for e in escapes:
+#             value = value.replace(e[1],e[0])
+#         return value
+
+#     for e in escapes:
+#         sql_qs = QuotedString("'", esc_quote="''")
+#         quote = sql_qs.search_string(value)
+#         if len(quote) > 0:
+#             quote = quote.asList()
+#             # print(f"quote: {quote}")
+#             for q in quote:
+#                 if len(q) == 1:
+#                     q = q[0]
+#                 esc = q.replace(e[0],e[1])
+#                 value = value.replace(q,esc)
+#     # print(sql_qs.search_string(value))
+#     return value
+
 
 
 
