@@ -27,7 +27,16 @@ import utils.string_utils as _csu
 # from utils.string_utils.string_format import to_screaming_snake as _csu.to_screaming_snake
 # from utils.string_utils.string_format import to_title_case as _csu.to_title_case
 
-from utils.random_utils.rand_generation import rand
+# from utils.random_utils.rand_generation import rand as _rand
+# include the random generators that produce strings.
+import utils.random_utils.rand_generation as _rand
+gender = _rand.gender
+user = _rand.user
+url = _rand.url
+email = _rand.email
+phone = _rand.phone
+abstract_name = _rand.abstract_name
+text = _rand.text
 
 def to_hash(value):
     '''
